/*
 * TempMonitor: 4-channel temperature monitor/logger
 *
 * Inputs : 4 Dallas Semiconductor DS18B20 one-wire temperature sensors
 *          Switch input - C or F
 *
 * Outputs : EA DOGM162 LCD Display - SPI interface
 *           Serial logging output
 *
 * Displays temperature from 4 sensors on line 1 of the display.  Displays the
 * delta between the lowest (assumed to be ambient) and the other sensors on
 * line 2.  Temperatures may either be in C or F.  Logs a timestamp
 * (milleseconds since starting) and the 4 temperature values in floating point
 * (Celsius) to the serial port at 9600 baud for each sample.
 *
 * IO Assignment
 *   D0 - Serial RX
 *   D1 - Serial TX
 *   D2 - One-wire interface (supports parasitic power w/ ext 4.7 kohm)
 *   D3 - Unit selection switch (0 = F, 1 = C) - uses internal pull-up
 *   D4 - LCD RS signal
 *   D10 - LCD CS signal (active low)
 *   D11 - LCD SI (MOSI)
 *   D13 - LCD CLK (SCLK)
 *
 * Copyright (c) 2011 Dan Julio.  All rights reserved.
 *
 * Licensed under the Creative Commons Attribution License.  just let them know
 * where you got the code.
 */
#include <LCDdogmSPI.h>
#include <OneWire.h>
#include <DallasTemperature.h>

// IO Pin assignments
#define ONE_WIRE_PIN 2
#define SEL_SW_PIN 3
#define RS_PIN 4
#define CSB_PIN 10

// LCD definitions
#define LCD_LINES 2
#define T1_COL 0
#define T2_COL 4
#define T3_COL 8
#define T4_COL 12
#define TIME_COL 0
#define UNIT_COL 15

// Number of bits of resolution
#define DEV_RESOLUTION 12

// Variables
int i;
int NumDevices;
float TempC[4]; // Floating point temperatures (Celsius) 
int IntTemp[4];
int TempCols[4] = {T1_COL, T2_COL, T3_COL, T4_COL};
unsigned long CurTime;
DeviceAddress devAddr;
boolean Enabled = true;
boolean DispC;  // 1 = display C, 0 = display F

// Object instantiations
LCDdogmSPI lcd = LCDdogmSPI(LCD_LINES, CSB_PIN, RS_PIN);
OneWire oneWire(ONE_WIRE_PIN);
DallasTemperature sensors(&oneWire);

void setup() {
  // Initialize the LCD
  lcd.init();
  lcd.cursorTo(0,0);
  lcd.println("TempMonitor v1.0");
  
  // Initialize the switch input
  pinMode(SEL_SW_PIN, INPUT);
  digitalWrite(SEL_SW_PIN, HIGH); // Enable the pull-up

  // Initialize the serial interface
  Serial.begin(9600);
  
  // Initialize the onewire system and locate devices
  lcd.cursorTo(1,0);
  lcd.println("Initializing...");
  sensors.begin();
  
  // Make sure we found all devices
  NumDevices = sensors.getDeviceCount();
  if (NumDevices == 4) {
    // Set the resolution to 12 bits
    for (i=0; i<4; i++) {
      (void) sensors.getAddress(devAddr, i);
      sensors.setResolution(devAddr, DEV_RESOLUTION);
    }
  } else {
    // Error out with some useful information
    Enabled = false;
    lcd.clear();
    lcd.println("Error: Dev Count");
    lcd.cursorTo(1,2);
    DispLcdTemp(NumDevices);
    lcd.cursorTo(1,6);
    lcd.println("Devices");
  }
  
  // Initiate the first temperature reading
  sensors.requestTemperatures();
  
  // Blank display for normal operating conditions
  if (Enabled) {
    lcd.clear();
  }
}

void loop(void) {
  CurTime = millis();
  if (Enabled) {
    // Get the current units to display
    DispC = digitalRead(SEL_SW_PIN);
      
    // Get the current temperatures
    for (i=0; i<4; i++) {
      (void) sensors.getAddress(devAddr, i);
      TempC[i] = sensors.getTempC(devAddr);
    }
      
    // Display the temperatures
    DisplayTemperatures();
    DisplayUnits();
    DisplayDeltas();
      
    // Log information to the serial port
    LogTimestamp();
    LogTemperatures();
      
    // Initiate the next reading
    sensors.requestTemperatures();
  }
}

// Display line 1 with current temperatures
//  Load IntTemp for computing deltas
void DisplayTemperatures(void) {  
  for (i=0; i<4; i++) {
    if (DispC) {
      IntTemp[i] = round(TempC[i]);
    } else {
      IntTemp[i] = round(sensors.toFahrenheit(TempC[i]));
    }
    lcd.cursorTo(0, TempCols[i]);
    DispLcdTemp(IntTemp[i]);
  }
}

// Display delta values on line 2
void DisplayDeltas(void) {
  int minTemp = 999;
  int delta;
  
  // Select the current minimum temperature
  for (i=0; i<4; i++) {
    if (minTemp > IntTemp[i])
      minTemp = IntTemp[i];
  }
  
  // Display the deltas
  for (i=0; i<4; i++) {
    delta = IntTemp[i] - minTemp;
    lcd.cursorTo(1, TempCols[i]);
    DispLcdTemp(delta);
  }
}

void DisplayUnits(void) {
  lcd.cursorTo(0, UNIT_COL);
  if (DispC) {
    lcd.print('C');
  } else {
    lcd.print('F');
  }
}

void LogTimestamp(void) {
  Serial.print(CurTime);
}

void LogTemperatures(void) {
  for (int i=0; i<4; i++) {
    Serial.print(" ");
    Serial.print(TempC[i]);
  }
  Serial.println("");
}

// Fast converter : Handles -99 - +999 at the current LCD cursor
void DispLcdTemp(int val) {
  int temp;
  boolean rbi = false;
  
  if (val < 0) {
    lcd.print('-');
    val = -val;
  } else {
    if (val >= 100) {
      temp = val / 100;
      lcd.print('0' + temp);
      val = val - (temp * 100);
      rbi = true; // Ensure something is displayed for tens
    } else {
      lcd.print(' ');
    }
  }
  
  if ((val >= 10) || rbi) {
    temp = val / 10;
    lcd.print('0' + temp);
    val = val - (temp * 10);
  } else {
    lcd.print(' ');
  }
  
  lcd.print('0' + val);
}
